==> sigaddons-7.0.21.12.14.tar.gz: sigaddons-7.0.21.12.14/sigaddons/fiscal/inherit/adm_pessoa.py <==
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from nfe.nf_e400 import nfe_400
from datetime import datetime


def remove_accents(text):
    from unicodedata import normalize, category
    unicode_string = unicode(text, 'utf-8') if type(text) == str else text
    text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

    return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])


class adm_pessoa(osv.Model):
    _inherit = 'adm.pessoa'

    def onchange_endereco_nf(self, cr, uid, ids, endereco_nf_id, context=None):
        """
        Verifica se o endereco selecionado e um endereco válido para Nota Fiscal

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param endereco_nf_id: {int} ID do endereco selecionado.
        @return: endereco selecionado ou mensagem de erro da verificacao do en-
        dereco.
        """
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pessoa_endereco = self.pool.get('adm.pessoa.endereco')

        if endereco_nf_id:
            msg = obj_pessoa_endereco.valida_endereco_nf(cr, uid, endereco_nf_id, context=context)

            if msg:
                return {
                    'value': {'endereco_nf_id': False},
                    'warning': {'title': _('Atencao'), 'message': msg}
                }

        return {'value': {'endereco_nf_id': endereco_nf_id}}

    def carregar_informacao_cnpj_sefaz(self, cr, uid, pessoa, uf, context=None):
        context = context and context.copy() or {}
        context.update({'busca_cnpj_sefaz': True})
        browse_pessoa = self.browse(cr, uid, pessoa, context)
        cpf_cnpj = (browse_pessoa.cpf_cnpj or '').replace('.', '').replace('-', '').replace('/', '')
        if not cpf_cnpj:
            raise osv.except_osv(_('Atencao!'),
                                 _('Destinatario sem informacao de CPF ou CNPJ.'))
        parametro_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', [], context=context)
        if datetime.strptime(
                parametro_fiscal.get('data_validade'), '%Y-%m-%d %H:%M:%S') < datetime.now():
            raise osv.except_osv(_('Atencao!'),
                                 _('Certificado digital com validade vencida.'))
        try:
            nfe = nfe_400(certify=parametro_fiscal.get('certificado_conteudo'),
                          key=parametro_fiscal.get('chave_conteudo'),
                          environment=parametro_fiscal.get('fispar_nfe_ambiente'), state=uf,
                          contingency=parametro_fiscal.get('fispar_nfe_contingencia'))
            processo, arquivos = nfe.consultar_cadastro_contribuinte(cpf_cnpj, '')
        except IOError:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))
        except Exception as erro:
            if str(erro).find('certificate expired') >= 1:
                erro = _('Certificado digital com validade vencida.')
            else:
                erro = _('Consulta nao implementada para este estado.')
            raise osv.except_osv(_('Atencao!'), erro)
        else:
            if not hasattr(processo.resposta, 'infCons') or \
                    not processo.resposta.infCons.cStat.valor:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha ao tentar conectar servidor da SEFAZ. %s - %s') % (
                                         processo.resposta.status,
                                         remove_accents(processo.resposta.reason)))
            elif processo.resposta.infCons.cStat.valor in (u'111', u'112'):
                info_cadastro = filter(
                    lambda f: f.CNPJ.valor == cpf_cnpj, processo.resposta.infCons.infCad)
            else:
                if processo.resposta.infCons.cStat.valor == u'265':
                    processo.resposta.infCons.xMotivo.valor += u' ou UF nao dispoe do servico de ' \
                                                               u'Conculsta Cadastro.'
                raise osv.except_osv(_('Atencao!'),
                                     '%s - %s' % (processo.resposta.infCons.cStat.valor,
                                                  processo.resposta.infCons.xMotivo.valor))

            if not info_cadastro:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao foi possivel carregar informacoes para o CNPJ '
                                       'informado. Verifique se informou o estado correto.'))
            info_cadastro = info_cadastro[0]
            vals = dict(name=info_cadastro.xNome.valor, fantasia_apelido=info_cadastro.xFant.valor,
                        admpes_jur_ie=info_cadastro.IE.valor,
                        admpes_jur_data_fundacao=str(info_cadastro.dIniAtiv.valor))
            cnae_id = self.pool.get('adm.cnae').search(
                cr, uid, [('admcna_codigo', '=', str(info_cadastro.CNAE.valor))], context=context)
            if cnae_id:
                vals = dict(vals, admpes_jur_cnae_id=cnae_id[0])

            if info_cadastro.xRegApur.valor == 'DEBITO E CREDITO':
                vals = dict(vals, admpes_jur_regime_tributario='R')

            self.write(cr, uid, pessoa, vals, context)
            self.pool.get('adm.pessoa.endereco').str2address(
                cr, uid, pessoa, 'Brasil', uf, info_cadastro.ender.xMun.valor,
                info_cadastro.ender.xBairro.valor, info_cadastro.ender.xLgr.valor,
                str(info_cadastro.ender.CEP.valor).zfill(8), info_cadastro.ender.nro.valor,
                IntCodigoIBGE=info_cadastro.ender.cMun.valor, context=context)

        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'endereco_nf_id' in vals and 'endereco_ids' in vals:
            if not vals['endereco_nf_id']:
                for x in vals['endereco_ids']:
                    if x[1] and x[0] == 1:
                        vals['endereco_nf_id'] = x[1]
                        break
                if not vals['endereco_nf_id']:
                    context.update({'sem_endereco_nf_pessoa': True})
        res = super(adm_pessoa, self).create(cr, uid, vals, context)
        adm_pessoa_dados = self.read(cr, uid, res, ['endereco_ids'], context=context)
        if isinstance(adm_pessoa_dados, list):
            adm_pessoa_dados = adm_pessoa_dados[0]
        endereco_ids = adm_pessoa_dados.get('endereco_ids')
        if endereco_ids:
            self.write(cr, uid, res, {'endereco_nf_id': endereco_ids[0]}, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            id_f = ids[0]
        else:
            id_f = ids
        if not 'endereco_nf_id' in vals:
            adm_pessoa_dados = self.read(cr, uid, id_f, ['endereco_nf_id'], context=context)
            if isinstance(adm_pessoa_dados, list):
                adm_pessoa_dados = adm_pessoa_dados[0]
            endereco_nf = adm_pessoa_dados.get('endereco_nf_id')
        else:
            endereco_nf = vals['endereco_nf_id']
        if not endereco_nf:
            context.update({'sem_endereco_nf_pessoa': id_f})
        res = super(adm_pessoa, self).write(cr, uid, ids, vals, context=context)
        return res

    def admpes_get_cobranca_endereco_id(self, cr, uid, pessoa_id, context=None):
        """
        O método encontra o registro (id) da tabela adm_pessoa_endereco para a pessoa_id passada, na
        seguite ordem: 1 - Endereco tipo cobranca, 2 - Endereco para NF e 3 - Primeiro encontrado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: id da tabela adm_pessoa_endereco, caso encontre o registro. Senao, Falso.
        :rtype: int
        """
        context = context and context.copy() or {}
        brw_adm_pessoa = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id, context)
        brw_adm_pessoa_endereco = brw_adm_pessoa.endereco_ids
        cobranca_endereco_ids = filter(lambda end: end.endereco_tipo_id.name == 'Cobranca',
                                       brw_adm_pessoa_endereco)
        endereco_id = False
        if cobranca_endereco_ids:
            endereco_id = cobranca_endereco_ids[0].id
        elif brw_adm_pessoa.endereco_nf_id:
            endereco_id = brw_adm_pessoa.endereco_nf_id.id
        elif brw_adm_pessoa_endereco:
            endereco_id = brw_adm_pessoa_endereco[0].id
        return endereco_id

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

==> sigaddons-7.0.2310.2.tar.gz: sigaddons-7.0.2310.2/sigaddons/fiscal/inherit/adm_pessoa.py <==
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from nfe.nf_e400 import nfe_400
from datetime import datetime


def remove_accents(text):
    from unicodedata import normalize, category
    unicode_string = unicode(text, 'utf-8') if type(text) == str else text
    text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

    return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])


class adm_pessoa(osv.Model):
    _inherit = 'adm.pessoa'

    def onchange_endereco_nf(self, cr, uid, ids, endereco_nf_id, context=None):
        """
        Verifica se o endereco selecionado e um endereco válido para Nota Fiscal

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param endereco_nf_id: {int} ID do endereco selecionado.
        @return: endereco selecionado ou mensagem de erro da verificacao do en-
        dereco.
        """
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pessoa_endereco = self.pool.get('adm.pessoa.endereco')

        if endereco_nf_id:
            msg = obj_pessoa_endereco.valida_endereco_nf(cr, uid, endereco_nf_id, context=context)

            if msg:
                return {
                    'value': {'endereco_nf_id': False},
                    'warning': {'title': _('Atencao'), 'message': msg}
                }

        return {'value': {'endereco_nf_id': endereco_nf_id}}

    def carregar_informacao_cnpj_sefaz(self, cr, uid, pessoa, uf, context=None):
        context = context and context.copy() or {}
        context.update({'busca_cnpj_sefaz': True})
        browse_pessoa = self.browse(cr, uid, pessoa, context)
        cpf_cnpj = (browse_pessoa.cpf_cnpj or '').replace('.', '').replace('-', '').replace('/', '')
        if not cpf_cnpj:
            raise osv.except_osv(_('Atencao!'),
                                 _('Destinatario sem informacao de CPF ou CNPJ.'))
        parametro_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', [], context=context)
        if datetime.strptime(
                parametro_fiscal.get('data_validade'), '%Y-%m-%d %H:%M:%S') < datetime.now():
            raise osv.except_osv(_('Atencao!'),
                                 _('Certificado digital com validade vencida.'))
        try:
            nfe = nfe_400(certify=parametro_fiscal.get('certificado_conteudo'),
                          key=parametro_fiscal.get('chave_conteudo'),
                          environment=parametro_fiscal.get('fispar_nfe_ambiente'), state=uf,
                          contingency=parametro_fiscal.get('fispar_nfe_contingencia'))
            processo, arquivos = nfe.consultar_cadastro_contribuinte(cpf_cnpj, '')
        except IOError:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))
        except Exception as erro:
            if str(erro).find('certificate expired') >= 1:
                erro = _('Certificado digital com validade vencida.')
            else:
                erro = _('Consulta nao implementada para este estado.')
            raise osv.except_osv(_('Atencao!'), erro)
        else:
            if not hasattr(processo.resposta, 'infCons') or not processo.resposta.infCons.cStat.valor:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha ao tentar conectar servidor da SEFAZ. \n%s - %s') % (
                                         processo.resposta.status,
                                         remove_accents(processo.resposta.reason)))
            elif processo.resposta.infCons.cStat.valor in (u'111', u'112'):
                info_cadastro = filter(
                    lambda f: f.CNPJ.valor == cpf_cnpj, processo.resposta.infCons.infCad)
            else:
                if processo.resposta.infCons.cStat.valor == u'265':
                    processo.resposta.infCons.xMotivo.valor += u' ou UF nao dispoe do servico de ' \
                                                               u'Conculsta Cadastro.'
                raise osv.except_osv(_('Atencao!'),
                                     '%s - %s' % (processo.resposta.infCons.cStat.valor,
                                                  processo.resposta.infCons.xMotivo.valor))

            if not info_cadastro:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao foi possivel carregar informacoes para o CNPJ '
                                       'informado. Verifique se informou o estado correto.'))
            info_cadastro = info_cadastro[0]
            vals = dict(name=info_cadastro.xNome.valor, fantasia_apelido=info_cadastro.xFant.valor,
                        admpes_jur_ie=info_cadastro.IE.valor,
                        admpes_jur_data_fundacao=str(info_cadastro.dIniAtiv.valor))
            cnae_id = self.pool.get('adm.cnae').search(
                cr, uid, [('admcna_codigo', '=', str(info_cadastro.CNAE.valor))], context=context)
            if cnae_id:
                vals = dict(vals, admpes_jur_cnae_id=cnae_id[0])

            if info_cadastro.xRegApur.valor == 'DEBITO E CREDITO':
                vals = dict(vals, admpes_jur_regime_tributario='R')

            self.write(cr, uid, pessoa, vals, context)
            self.pool.get('adm.pessoa.endereco').str2address(
                cr, uid, pessoa, 'Brasil', uf, info_cadastro.ender.xMun.valor,
                info_cadastro.ender.xBairro.valor, info_cadastro.ender.xLgr.valor,
                str(info_cadastro.ender.CEP.valor).zfill(8), info_cadastro.ender.nro.valor,
                IntCodigoIBGE=info_cadastro.ender.cMun.valor, context=context)

        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'endereco_nf_id' in vals and 'endereco_ids' in vals:
            if not vals['endereco_nf_id']:
                for x in vals['endereco_ids']:
                    if x[1] and x[0] == 1:
                        vals['endereco_nf_id'] = x[1]
                        break
                if not vals['endereco_nf_id']:
                    context.update({'sem_endereco_nf_pessoa': True})
        res = super(adm_pessoa, self).create(cr, uid, vals, context)
        adm_pessoa_dados = self.read(cr, uid, res, ['endereco_ids'], context=context)
        if isinstance(adm_pessoa_dados, list):
            adm_pessoa_dados = adm_pessoa_dados[0]
        endereco_ids = adm_pessoa_dados.get('endereco_ids')
        if endereco_ids:
            self.write(cr, uid, res, {'endereco_nf_id': endereco_ids[0]}, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            id_f = ids[0]
        else:
            id_f = ids
        if not 'endereco_nf_id' in vals:
            adm_pessoa_dados = self.read(cr, uid, id_f, ['endereco_nf_id'], context=context)
            if isinstance(adm_pessoa_dados, list):
                adm_pessoa_dados = adm_pessoa_dados[0]
            endereco_nf = adm_pessoa_dados.get('endereco_nf_id')
        else:
            endereco_nf = vals['endereco_nf_id']
        if not endereco_nf:
            context.update({'sem_endereco_nf_pessoa': id_f})
        res = super(adm_pessoa, self).write(cr, uid, ids, vals, context=context)
        return res

    def admpes_get_cobranca_endereco_id(self, cr, uid, pessoa_id, context=None):
        """
        O método encontra o registro (id) da tabela adm_pessoa_endereco para a pessoa_id passada, na
        seguite ordem: 1 - Endereco tipo cobranca, 2 - Endereco para NF e 3 - Primeiro encontrado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: id da tabela adm_pessoa_endereco, caso encontre o registro. Senao, Falso.
        :rtype: int
        """
        context = context and context.copy() or {}
        brw_adm_pessoa = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id, context)
        brw_adm_pessoa_endereco = brw_adm_pessoa.endereco_ids
        cobranca_endereco_ids = filter(lambda end: end.endereco_tipo_id.name == 'Cobranca',
                                       brw_adm_pessoa_endereco)
        endereco_id = False
        if cobranca_endereco_ids:
            endereco_id = cobranca_endereco_ids[0].id
        elif brw_adm_pessoa.endereco_nf_id:
            endereco_id = brw_adm_pessoa.endereco_nf_id.id
        elif brw_adm_pessoa_endereco:
            endereco_id = brw_adm_pessoa_endereco[0].id
        return endereco_id

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
